#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <beryl-settings.h>
enum mod
{
	MODE_NONE,
	MODE_CONFLICTS='c',
	MODE_GET='g',
	DIVIDER,
	OPT_VERSION='v',
	OPT_PLUGIN='p',
	OPT_SETTING='s',
};

static void usage(void)
{
	fprintf(stderr,
			"Usage:\n"
			"\tberyl-settings-tool --get --plugin[=]NAME --setting[=]NAME\n"
			"\tberyl-settings-tool --conflicts[={key|edge|button}] [--plugin[=]NAME --setting[=]NAME]\n"
			);
	exit(0);
}

int main(int argc, char ** argv)
{
	int mode=MODE_NONE;
	char sopts[]="cgv";
	gchar * plugin=NULL;
	gchar * setting=NULL;
	BerylSettingConflictType cmode = BERYL_SETTING_CONFLICT_TYPE_ANY;
	struct option lopts[]={
		{"version",0,0,OPT_VERSION},
		{"conflicts",2,0,MODE_CONFLICTS},
		{"get",0,&mode,MODE_GET},
		{"plugin",1,0,OPT_PLUGIN},
		{"setting",1,0,OPT_SETTING},
		{0,0,0,0}
	};
	while(1)
	{
		int c;
		c=getopt_long(argc,argv,sopts,lopts,NULL);
		if (c==-1)
			break;
		switch(c)
		{
			case MODE_CONFLICTS:
				mode=c;
				if (optarg)
				{
					if (g_strcasecmp(optarg,"key")==0)
						cmode=BERYL_SETTING_CONFLICT_TYPE_KEY;
					else if (g_strcasecmp(optarg,"button")==0)
						cmode=BERYL_SETTING_CONFLICT_TYPE_BUTTON;
					else if (g_strcasecmp(optarg,"edge")==0)
						cmode=BERYL_SETTING_CONFLICT_TYPE_EDGE;
					else
						usage();
				}
				break;
			case OPT_VERSION:
				printf("beryl-settings-tool (");
				printf(PACKAGE_STRING ")\n");
				return 0;
			case OPT_PLUGIN:
				if (!optarg)
					g_error("foo");
				plugin=g_strdup(optarg);
				break;
			case OPT_SETTING:
				if (!optarg)
					g_error("bar");
				setting=g_strdup(optarg);
				break;
			default:
				break;
		}
	}
	if (mode==MODE_NONE)
		usage();
	BerylSettingsContext * ctx=beryl_settings_context_new();
	switch(mode)
	{
		case MODE_CONFLICTS:
			{
				BerylSettingsPlugin * p;
				BerylSetting * s=NULL;
				if (plugin && setting)
				{
					if (strcmp(plugin,"_")==0) plugin=NULL;
					p=beryl_settings_context_find_plugin(ctx,plugin);
					if (p)
					{
						s=beryl_settings_plugin_find_setting(p,setting,TRUE);
						if (!s)
							s=beryl_settings_plugin_find_setting(p,setting,FALSE);
					}
				}
				GSList * conf;
				if (s)
					conf = beryl_settings_context_find_conflicts_for_setting(ctx,s,cmode);
				else
					conf = beryl_settings_context_find_conflicts(ctx,cmode);
				GSList * n;
				for(n=conf;n;n=n->next)
				{
					gchar * bnd;
					gchar * mdt;
					gchar * kdt;
					int mds;
					GSList * stn = beryl_setting_conflict_get_settings((BerylSettingConflict *)n->data);
					BerylSettingValue * bsv = beryl_setting_get_primary_value((BerylSetting *)stn->data);
					if (!stn)
						g_error("PATHOLOGIOCAL");
					switch(beryl_setting_conflict_get_type((BerylSettingConflict *)n->data))
					{
						case BERYL_SETTING_CONFLICT_TYPE_KEY:
							beryl_setting_value_get_keymods(bsv,&mds);
							mdt=beryl_settings_mods_to_string(mds);
							beryl_setting_value_get_keysym(bsv,&mds);
							if (!(kdt=XKeysymToString(mds)))
								kdt="None";
							bnd=g_strdup_printf("Key Binding %s%s",mdt,kdt);
							g_free(mdt);
							break;
						case BERYL_SETTING_CONFLICT_TYPE_BUTTON:
							beryl_setting_value_get_buttonmods(bsv,&mds);
							mdt=beryl_settings_mods_to_string(mds);
							beryl_setting_value_get_button(bsv,&mds);
							if (mds==0)
								kdt="Any";
							else
								kdt=g_strdup_printf("Button%d",mds);
							bnd=g_strdup_printf("Button Binding %s%s",mdt,kdt);
							g_free(kdt);
							g_free(mdt);
							break;
						case BERYL_SETTING_CONFLICT_TYPE_EDGE:
							bnd=g_strdup("An Edge Binding");	
							break;
						default:
							break;
					}
					fprintf(stdout,"These settings conflict over %s\n",bnd);
					g_free(bnd);
					for(;stn;stn=stn->next)
					{
						BerylSetting * s = stn->data;
						fprintf(stdout,"	%s/%s\n",beryl_settings_plugin_get_name(beryl_setting_get_plugin(s)),beryl_setting_get_name(s));
					}
				}
			}
			break;
		case MODE_GET:
			{
				if (!plugin)
					usage();
				if (!setting)
					usage();
				if (strcmp(plugin,"_")==0)
					plugin=NULL;
				BerylSettingsPlugin * p = beryl_settings_context_find_plugin(ctx,plugin);
				if (!p)
				{
					fprintf(stdout,"Plugin %s not found.\n",plugin);
					return 1;
				}
				BerylSetting * s = beryl_settings_plugin_find_setting(p,setting,TRUE);
				if (!s)
					s = beryl_settings_plugin_find_setting(p,setting,FALSE);
				if (!s)
				{
					fprintf(stdout,"Setting %s not found in plugin %s.\n",setting,plugin);
					return 1;
				}
				BerylSettingValue * v = beryl_setting_get_primary_value(s);
				switch(beryl_setting_get_type(s))
				{
					case BERYL_SETTING_TYPE_BOOL:
						{
							gboolean b;
							beryl_setting_value_get_bool(v,&b);
							fprintf(stdout,b?"TRUE\n":"FALSE\n");
						}
						break;
					case BERYL_SETTING_TYPE_STRING:
						{
							const gchar * s;
							beryl_setting_value_get_string(v,&s);
							fprintf(stdout,"%s\n",s);
						}
						break;
					case BERYL_SETTING_TYPE_INT:
						{
							gint i;
							beryl_setting_value_get_int(v,&i);
							fprintf(stdout,"%d\n",i);
						}
						break;
					case BERYL_SETTING_TYPE_FLOAT:
						{
							gdouble d;
							beryl_setting_value_get_float(v,&d);
							fprintf(stdout,"%lf\n",d);
						}
						break;
					case BERYL_SETTING_TYPE_BINDING:
						{
							gint kbt;
							gchar * tmods;
							gchar * tkb;
							gboolean b;
							beryl_setting_get_can_set_key(s,&b);
							if (b)
							{
								beryl_setting_value_get_key_enabled(v,&b);
								if (b)
								{
									beryl_setting_value_get_keymods(v,&kbt);
									tmods=beryl_settings_mods_to_string(kbt);
									beryl_setting_value_get_keysym(v,&kbt);
									tkb=XKeysymToString(kbt);
									if (!tkb) tkb="None";
									fprintf(stdout,"Key:%s%s\n",tmods,tkb);
									g_free(tmods);
								}
								else
									fprintf(stdout,"Key:Disabled\n");
							}
							beryl_setting_get_can_set_button(s,&b);
							if (b)
							{
								beryl_setting_value_get_button_enabled(v,&b);
								if (b)
								{
									beryl_setting_value_get_buttonmods(v,&kbt);
									tmods=beryl_settings_mods_to_string(kbt);
									beryl_setting_value_get_button(v,&kbt);
									if (kbt==0)
										tkb=g_strdup("Any");
									else
										tkb=g_strdup_printf("Button%d",kbt);
									fprintf(stdout,"Button:%s%s\n",tmods,tkb);
									g_free(tmods);
									g_free(tkb);
								}
								else
									fprintf(stdout,"Button:Disabled\n");
							}
							beryl_setting_get_can_set_edgemask(s,&b);
							if (b)
							{
								int edm;
								beryl_setting_value_get_edgemask(v,&edm);
								fprintf(stdout,"Edgemask:%d\n",edm);
							}
							beryl_setting_get_can_set_bell(s,&b);
							if (b)
							{
								beryl_setting_value_get_bell(v,&b);
								fprintf(stdout,"On Bell:%s\n",b?"TRUE":"FALSE");
							}
						}
						break;
					case BERYL_SETTING_TYPE_COLOR:
						{
							BerylSettingColorValue c;
							beryl_setting_value_get_color(v,&c);
							fprintf(stdout,"Color (R,G,B,A): (%lf,%lf,%lf,%lf)\n",c.color.red/(gdouble)0xFFFF,
									c.color.green/(gdouble)0xFFFF,c.color.blue/(gdouble)0xFFFF,c.color.alpha/(gdouble)0xFFFF);
						}
						break;
					case BERYL_SETTING_TYPE_LIST:
						{
							BerylSettingType t;
							beryl_setting_get_list_type(s,&t);
							if (t==BERYL_SETTING_TYPE_STRING)
							{
								GSList * l;
								beryl_setting_value_get_value_list(v,&l);
								for (;l;l=l->next)
								{
									const gchar * s;
									beryl_setting_value_get_string((BerylSettingValue *)l->data,&s);
									fprintf(stdout,"%s\n",s);
								}
							}
						}
						break;
					default:
						fprintf(stdout,"Setting type not supported.\n");
				}
			}
		default:
			break;
	}
	return 0;
}
